==> vavoom-1.19.tar.bz2: vavoom-1.19/source/sc_man.cpp <==
//**************************************************************************
//**
//**	##   ##    ##    ##   ##   ####     ####   ###     ###
//**	##   ##  ##  ##  ##   ##  ##  ##   ##  ##  ####   ####
//**	 ## ##  ##    ##  ## ##  ##    ## ##    ## ## ## ## ##
//**	 ## ##  ########  ## ##  ##    ## ##    ## ##  ###  ##
//**	  ###   ##    ##   ###    ##  ##   ##  ##  ##       ##
//**	   #    ##    ##    #      ####     ####   ##       ##
//**
//**	$Id: sc_man.cpp,v 1.8 2005/07/03 12:07:37 dj_jl Exp $
//**
//**	Copyright (C) 1999-2002 Jnis Legzdi
//**
//**	This program is free software; you can redistribute it and/or
//**  modify it under the terms of the GNU General Public License
//**  as published by the Free Software Foundation; either version 2
//**  of the License, or (at your option) any later version.
//**
//**	This program is distributed in the hope that it will be useful,
//**  but WITHOUT ANY WARRANTY; without even the implied warranty of
//**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//**  GNU General Public License for more details.
//**
//**************************************************************************

// HEADER FILES ------------------------------------------------------------

#include "gamedefs.h"

// MACROS ------------------------------------------------------------------

#define MAX_STRING_SIZE 64
#define ASCII_COMMENT (';')
#define ASCII_QUOTE (34)
#define LUMP_SCRIPT 1
#define FILE_ZONE_SCRIPT 2
#define LUMP_NUM_SCRIPT 3

// TYPES -------------------------------------------------------------------

// EXTERNAL FUNCTION PROTOTYPES --------------------------------------------

// PUBLIC FUNCTION PROTOTYPES ----------------------------------------------

// PRIVATE FUNCTION PROTOTYPES ---------------------------------------------

static void CheckOpen(void);
static void OpenScript(const char *name, int LumpNum, int type);

// EXTERNAL DATA DECLARATIONS ----------------------------------------------

// PUBLIC DATA DEFINITIONS -------------------------------------------------

char 	*sc_String;
int 	sc_Number;
double 	sc_Float;
int 	sc_Line;
boolean sc_End;
boolean sc_Crossed;

// PRIVATE DATA DEFINITIONS ------------------------------------------------

static char 	ScriptName[16];
static char 	*ScriptBuffer;
static char 	*ScriptPtr;
static char 	*ScriptEndPtr;
static char 	StringBuffer[MAX_STRING_SIZE];
static boolean 	ScriptOpen = false;
static int 		ScriptSize;
static boolean 	AlreadyGot = false;

// CODE --------------------------------------------------------------------

//==========================================================================
//
//	SC_Open
//
//==========================================================================

void SC_Open(const char *name)
{
	guard(SC_Open);
	char filename[MAX_OSPATH];

	if (fl_devmode && FL_FindFile(va("scripts/%s.txt", name), filename))
	{
		SC_OpenFile(filename);
	}
	else
	{
		SC_OpenLump(name);
	}
	unguard;
}

//==========================================================================
//
// SC_OpenLump
//
// Loads a script (from the WAD files) and prepares it for parsing.
//
//==========================================================================

void SC_OpenLump(const char *name)
{
	guard(SC_OpenLump);
	OpenScript(name, -1, LUMP_SCRIPT);
	unguard;
}

//==========================================================================
//
// SC_OpenLumpNum
//
// Loads a script (from the WAD files) and prepares it for parsing.
//
//==========================================================================

void SC_OpenLumpNum(int LumpNum)
{
	guard(SC_OpenLumpNum);
	OpenScript(W_LumpName(LumpNum), LumpNum, LUMP_NUM_SCRIPT);
	unguard;
}

//==========================================================================
//
// SC_OpenFile
//
// Loads a script (from a file) and prepares it for parsing.  Uses the
// zone memory allocator for memory allocation and de-allocation.
//
//==========================================================================

void SC_OpenFile(const char *name)
{
	guard(SC_OpenFile);
	OpenScript(name, -1, FILE_ZONE_SCRIPT);
	unguard;
}

//==========================================================================
//
// OpenScript
//
//==========================================================================

static void OpenScript(const char *name, int LumpNum, int type)
{
	guard(OpenScript);
	SC_Close();
	if (type == LUMP_SCRIPT)
	{
		// Lump script
		ScriptBuffer = (char *)W_CacheLumpName(name, PU_HIGH);
		ScriptSize = W_LumpLength(W_GetNumForName(name));
		strcpy(ScriptName, name);
	}
	else if (type == LUMP_NUM_SCRIPT)
	{
		// Lump num script
		ScriptBuffer = (char*)W_CacheLumpNum(LumpNum, PU_HIGH);
		ScriptSize = W_LumpLength(LumpNum);
		strcpy(ScriptName, name);
	}
	else if (type == FILE_ZONE_SCRIPT)
	{
		// File script
		ScriptSize = M_ReadFile(name, (byte **)&ScriptBuffer);
		FL_ExtractFileBase(name, ScriptName);
	}
	ScriptPtr = ScriptBuffer;
	ScriptEndPtr = ScriptPtr+ScriptSize;
	sc_Line = 1;
	sc_End = false;
	ScriptOpen = true;
	sc_String = StringBuffer;
	AlreadyGot = false;
	unguard;
}

//==========================================================================
//
// SC_Close
//
//==========================================================================

void SC_Close(void)
{
	guard(SC_Close);
	if (ScriptOpen)
	{
		Z_Free(ScriptBuffer);
		ScriptOpen = false;
	}
	unguard;
}

//==========================================================================
//
// SC_GetString
//
//==========================================================================

boolean SC_GetString(void)
{
	guard(SC_GetString);
	char 	*text;
	boolean foundToken;

	CheckOpen();
	if (AlreadyGot)
	{
		AlreadyGot = false;
		return true;
	}
	foundToken = false;
	sc_Crossed = false;
	if (ScriptPtr >= ScriptEndPtr)
	{
		sc_End = true;
		return false;
	}
	while (foundToken == false)
	{
		while (*ScriptPtr <= 32)
		{
			if (ScriptPtr >= ScriptEndPtr)
			{
				sc_End = true;
				return false;
			}
			if (*ScriptPtr++ == '\n')
			{
				sc_Line++;
				sc_Crossed = true;
			}
		}
		if (ScriptPtr >= ScriptEndPtr)
		{
			sc_End = true;
			return false;
		}
		if (*ScriptPtr == ASCII_COMMENT ||
			(ScriptPtr[0] == '/' && ScriptPtr[1] == '/'))
		{
			// Skip comment
			while (*ScriptPtr++ != '\n')
			{
				if (ScriptPtr >= ScriptEndPtr)
				{
					sc_End = true;
					return false;
	
				}
			}
			sc_Line++;
			sc_Crossed = true;
		}
		else
		{
			// Found a token
			foundToken = true;
		}
	}
	text = sc_String;
	if (*ScriptPtr == ASCII_QUOTE)
	{ // Quoted string
		ScriptPtr++;
		while (*ScriptPtr != ASCII_QUOTE)
		{
			*text++ = *ScriptPtr++;
			if(ScriptPtr == ScriptEndPtr
				|| text == &sc_String[MAX_STRING_SIZE-1])
			{
				break;
			}
		}
		ScriptPtr++;
	}
	else
	{
		// Normal string
		while ((*ScriptPtr > 32) && !(*ScriptPtr == ASCII_COMMENT ||
			(ScriptPtr[0] == '/' && ScriptPtr[1] == '/')))
		{
			*text++ = *ScriptPtr++;
			if(ScriptPtr == ScriptEndPtr
				|| text == &sc_String[MAX_STRING_SIZE-1])
			{
				break;
			}
		}
	}
	*text = 0;
	return true;
	unguard;
}

//==========================================================================
//
// SC_MustGetString
//
//==========================================================================

void SC_MustGetString(void)
{
	guard(SC_MustGetString);
	if (SC_GetString() == false)
	{
		SC_ScriptError("Missing string.");
	}
	unguard;
}

//==========================================================================
//
// SC_MustGetStringName
//
//==========================================================================

void SC_MustGetStringName(const char *name)
{
	guard(SC_MustGetStringName);
	SC_MustGetString();
	if (SC_Compare(name) == false)
	{
		SC_ScriptError(NULL);
	}
	unguard;
}

//==========================================================================
//
//	SC_CheckNumber
//
//==========================================================================

boolean SC_CheckNumber(void)
{
	guard(SC_CheckNumber);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Number = strtol(sc_String, &stopper, 0);
		if (*stopper != 0)
		{
			SC_UnGet();
			return false;
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
// SC_GetNumber
//
//==========================================================================

boolean SC_GetNumber(void)
{
	guard(SC_GetNumber);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Number = strtol(sc_String, &stopper, 0);
		if (*stopper != 0)
		{
			Sys_Error("SC_GetNumber: Bad numeric constant \"%s\".\n"
				"Script %s, Line %d", sc_String, ScriptName, sc_Line);
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
// SC_MustGetNumber
//
//==========================================================================

void SC_MustGetNumber(void)
{
	guard(SC_MustGetNumber);
	if (SC_GetNumber() == false)
	{
		SC_ScriptError("Missing integer.");
	}
	unguard;
}

//==========================================================================
//
//	SC_CheckFloat
//
//==========================================================================

boolean SC_CheckFloat(void)
{
	guard(SC_CheckFloat);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Float = strtod(sc_String, &stopper);
		if (*stopper != 0)
		{
			SC_UnGet();
			return false;
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
//	SC_GetFloat
//
//==========================================================================

boolean SC_GetFloat(void)
{
	guard(SC_GetFloat);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Float = strtod(sc_String, &stopper);
		if (*stopper != 0)
		{
			Sys_Error("SC_GetFloat: Bad floating point constant \"%s\".\n"
				"Script %s, Line %d", sc_String, ScriptName, sc_Line);
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
//	SC_MustGetFloat
//
//==========================================================================

void SC_MustGetFloat(void)
{
	guard(SC_MustGetFloat);
	if (SC_GetFloat() == false)
	{
		SC_ScriptError("Missing float.");
	}
	unguard;
}

//==========================================================================
//
// SC_UnGet
//
// Assumes there is a valid string in sc_String.
//
//==========================================================================

void SC_UnGet(void)
{
	guard(SC_UnGet);
	AlreadyGot = true;
	unguard;
}

//==========================================================================
//
// SC_Check
//
// Returns true if another token is on the current line.
//
//==========================================================================

/*
boolean SC_Check(void)
{
	char *text;

	CheckOpen();
	text = ScriptPtr;
	if(text >= ScriptEndPtr)
	{
		return false;
	}
	while(*text <= 32)
	{
		if(*text == '\n')
		{
			return false;
		}
		text++;
		if(text == ScriptEndPtr)
		{
			return false;
		}
	}
	if(*text == ASCII_COMMENT)
	{
		return false;
	}
	return true;
}
*/

//==========================================================================
//
// SC_MatchString
//
// Returns the index of the first match to sc_String from the passed
// array of strings, or -1 if not found.
//
//==========================================================================

int SC_MatchString(const char **strings)
{
	guard(SC_MatchString);
	int i;

	for (i = 0; *strings != NULL; i++)
	{
		if (SC_Compare(*strings++))
		{
			return i;
		}
	}
	unguard;
	return -1;
}

//==========================================================================
//
// SC_MustMatchString
//
//==========================================================================

int SC_MustMatchString(const char **strings)
{
	guard(SC_MustMatchString);
	int i;

	i = SC_MatchString(strings);
	if (i == -1)
	{
		SC_ScriptError(NULL);
	}
	return i;
	unguard;
}

//==========================================================================
//
// SC_Compare
//
//==========================================================================

boolean SC_Compare(const char *text)
{
	guard(SC_Compare);
	return !stricmp(text, sc_String);
	unguard;
}

//==========================================================================
//
// SC_ScriptError
//
//==========================================================================

void SC_ScriptError(const char *message)
{
	guard(SC_ScriptError)
	const char* Msg = message ? message : "Bad syntax.";
	Sys_Error("Script error, \"%s\" line %d: %s", ScriptName,
		sc_Line, Msg);
	unguard;
}

//==========================================================================
//
// CheckOpen
//
//==========================================================================

static void CheckOpen(void)
{
	guard(CheckOpen);
	if (ScriptOpen == false)
	{
		Sys_Error("SC_ call before SC_Open().");
	}
	unguard;
}

//**************************************************************************
//
//	$Log: sc_man.cpp,v $
//	Revision 1.8  2005/07/03 12:07:37  dj_jl
//	Fixed opening of lump num scripts.
//	
//	Revision 1.7  2005/05/03 15:00:11  dj_jl
//	Moved switch list, animdefs enhancements.
//	
//	Revision 1.6  2002/01/07 12:16:43  dj_jl
//	Changed copyright year
//	
//	Revision 1.5  2001/10/12 17:31:13  dj_jl
//	no message
//	
//	Revision 1.4  2001/08/04 17:27:39  dj_jl
//	Added consts to script functions
//	
//	Revision 1.3  2001/07/31 17:16:31  dj_jl
//	Just moved Log to the end of file
//	
//	Revision 1.2  2001/07/27 14:27:54  dj_jl
//	Update with Id-s and Log-s, some fixes
//
//**************************************************************************

==> vavoom-1.20.tar.bz2: vavoom-1.20/source/sc_man.cpp <==
//**************************************************************************
//**
//**	##   ##    ##    ##   ##   ####     ####   ###     ###
//**	##   ##  ##  ##  ##   ##  ##  ##   ##  ##  ####   ####
//**	 ## ##  ##    ##  ## ##  ##    ## ##    ## ## ## ## ##
//**	 ## ##  ########  ## ##  ##    ## ##    ## ##  ###  ##
//**	  ###   ##    ##   ###    ##  ##   ##  ##  ##       ##
//**	   #    ##    ##    #      ####     ####   ##       ##
//**
//**	$Id: sc_man.cpp,v 1.10 2006/03/04 16:01:34 dj_jl Exp $
//**
//**	Copyright (C) 1999-2002 Jnis Legzdi
//**
//**	This program is free software; you can redistribute it and/or
//**  modify it under the terms of the GNU General Public License
//**  as published by the Free Software Foundation; either version 2
//**  of the License, or (at your option) any later version.
//**
//**	This program is distributed in the hope that it will be useful,
//**  but WITHOUT ANY WARRANTY; without even the implied warranty of
//**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//**  GNU General Public License for more details.
//**
//**************************************************************************

// HEADER FILES ------------------------------------------------------------

#include "gamedefs.h"

// MACROS ------------------------------------------------------------------

#define MAX_STRING_SIZE 64
#define ASCII_COMMENT (';')
#define ASCII_QUOTE (34)
#define LUMP_SCRIPT 1
#define FILE_ZONE_SCRIPT 2
#define LUMP_NUM_SCRIPT 3

// TYPES -------------------------------------------------------------------

// EXTERNAL FUNCTION PROTOTYPES --------------------------------------------

// PUBLIC FUNCTION PROTOTYPES ----------------------------------------------

// PRIVATE FUNCTION PROTOTYPES ---------------------------------------------

static void CheckOpen(void);
static void OpenScript(const char *name, int LumpNum, int type);

// EXTERNAL DATA DECLARATIONS ----------------------------------------------

// PUBLIC DATA DEFINITIONS -------------------------------------------------

char 	*sc_String;
int 	sc_Number;
double 	sc_Float;
int 	sc_Line;
boolean sc_End;
boolean sc_Crossed;

// PRIVATE DATA DEFINITIONS ------------------------------------------------

static VStr		ScriptName;
static char		*ScriptBuffer;
static char		*ScriptPtr;
static char		*ScriptEndPtr;
static char		StringBuffer[MAX_STRING_SIZE];
static bool		ScriptOpen = false;
static int		ScriptSize;
static bool		AlreadyGot = false;

// CODE --------------------------------------------------------------------

//==========================================================================
//
//	SC_Open
//
//==========================================================================

void SC_Open(const char *name)
{
	guard(SC_Open);
	VStr filename = FL_FindFile(va("scripts/%s.txt", name));
	if (fl_devmode && filename)
	{
		SC_OpenFile(*filename);
	}
	else
	{
		SC_OpenLump(name);
	}
	unguard;
}

//==========================================================================
//
// SC_OpenLump
//
// Loads a script (from the WAD files) and prepares it for parsing.
//
//==========================================================================

void SC_OpenLump(const char *name)
{
	guard(SC_OpenLump);
	OpenScript(name, -1, LUMP_SCRIPT);
	unguard;
}

//==========================================================================
//
// SC_OpenLumpNum
//
// Loads a script (from the WAD files) and prepares it for parsing.
//
//==========================================================================

void SC_OpenLumpNum(int LumpNum)
{
	guard(SC_OpenLumpNum);
	OpenScript(*W_LumpName(LumpNum), LumpNum, LUMP_NUM_SCRIPT);
	unguard;
}

//==========================================================================
//
// SC_OpenFile
//
// Loads a script (from a file) and prepares it for parsing.  Uses the
// zone memory allocator for memory allocation and de-allocation.
//
//==========================================================================

void SC_OpenFile(const char *name)
{
	guard(SC_OpenFile);
	OpenScript(name, -1, FILE_ZONE_SCRIPT);
	unguard;
}

//==========================================================================
//
// OpenScript
//
//==========================================================================

static void OpenScript(const char *name, int LumpNum, int type)
{
	guard(OpenScript);
	SC_Close();
	if (type == LUMP_SCRIPT)
	{
		// Lump script
		ScriptBuffer = (char*)W_CacheLumpName(VName(name, VName::AddLower8), PU_HIGH);
		ScriptSize = W_LumpLength(W_GetNumForName(VName(name, VName::AddLower8)));
		ScriptName = name;
	}
	else if (type == LUMP_NUM_SCRIPT)
	{
		// Lump num script
		ScriptBuffer = (char*)W_CacheLumpNum(LumpNum, PU_HIGH);
		ScriptSize = W_LumpLength(LumpNum);
		ScriptName = name;
	}
	else if (type == FILE_ZONE_SCRIPT)
	{
		// File script
		ScriptSize = M_ReadFile(name, (byte **)&ScriptBuffer);
		ScriptName = VStr(name).ExtractFileBase();
	}
	ScriptPtr = ScriptBuffer;
	ScriptEndPtr = ScriptPtr+ScriptSize;
	sc_Line = 1;
	sc_End = false;
	ScriptOpen = true;
	sc_String = StringBuffer;
	AlreadyGot = false;
	unguard;
}

//==========================================================================
//
// SC_Close
//
//==========================================================================

void SC_Close(void)
{
	guard(SC_Close);
	if (ScriptOpen)
	{
		Z_Free(ScriptBuffer);
		ScriptOpen = false;
	}
	unguard;
}

//==========================================================================
//
// SC_GetString
//
//==========================================================================

boolean SC_GetString(void)
{
	guard(SC_GetString);
	char 	*text;
	boolean foundToken;

	CheckOpen();
	if (AlreadyGot)
	{
		AlreadyGot = false;
		return true;
	}
	foundToken = false;
	sc_Crossed = false;
	if (ScriptPtr >= ScriptEndPtr)
	{
		sc_End = true;
		return false;
	}
	while (foundToken == false)
	{
		while (*ScriptPtr <= 32)
		{
			if (ScriptPtr >= ScriptEndPtr)
			{
				sc_End = true;
				return false;
			}
			if (*ScriptPtr++ == '\n')
			{
				sc_Line++;
				sc_Crossed = true;
			}
		}
		if (ScriptPtr >= ScriptEndPtr)
		{
			sc_End = true;
			return false;
		}
		if (*ScriptPtr == ASCII_COMMENT ||
			(ScriptPtr[0] == '/' && ScriptPtr[1] == '/'))
		{
			// Skip comment
			while (*ScriptPtr++ != '\n')
			{
				if (ScriptPtr >= ScriptEndPtr)
				{
					sc_End = true;
					return false;
	
				}
			}
			sc_Line++;
			sc_Crossed = true;
		}
		else
		{
			// Found a token
			foundToken = true;
		}
	}
	text = sc_String;
	if (*ScriptPtr == ASCII_QUOTE)
	{ // Quoted string
		ScriptPtr++;
		while (*ScriptPtr != ASCII_QUOTE)
		{
			*text++ = *ScriptPtr++;
			if(ScriptPtr == ScriptEndPtr
				|| text == &sc_String[MAX_STRING_SIZE-1])
			{
				break;
			}
		}
		ScriptPtr++;
	}
	else
	{
		// Normal string
		while ((*ScriptPtr > 32) && !(*ScriptPtr == ASCII_COMMENT ||
			(ScriptPtr[0] == '/' && ScriptPtr[1] == '/')))
		{
			*text++ = *ScriptPtr++;
			if(ScriptPtr == ScriptEndPtr
				|| text == &sc_String[MAX_STRING_SIZE-1])
			{
				break;
			}
		}
	}
	*text = 0;
	return true;
	unguard;
}

//==========================================================================
//
// SC_MustGetString
//
//==========================================================================

void SC_MustGetString(void)
{
	guard(SC_MustGetString);
	if (SC_GetString() == false)
	{
		SC_ScriptError("Missing string.");
	}
	unguard;
}

//==========================================================================
//
// SC_MustGetStringName
//
//==========================================================================

void SC_MustGetStringName(const char *name)
{
	guard(SC_MustGetStringName);
	SC_MustGetString();
	if (SC_Compare(name) == false)
	{
		SC_ScriptError(NULL);
	}
	unguard;
}

//==========================================================================
//
//	SC_CheckNumber
//
//==========================================================================

boolean SC_CheckNumber(void)
{
	guard(SC_CheckNumber);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Number = strtol(sc_String, &stopper, 0);
		if (*stopper != 0)
		{
			SC_UnGet();
			return false;
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
// SC_GetNumber
//
//==========================================================================

boolean SC_GetNumber(void)
{
	guard(SC_GetNumber);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Number = strtol(sc_String, &stopper, 0);
		if (*stopper != 0)
		{
			Sys_Error("SC_GetNumber: Bad numeric constant \"%s\".\n"
				"Script %s, Line %d", sc_String, *ScriptName, sc_Line);
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
// SC_MustGetNumber
//
//==========================================================================

void SC_MustGetNumber(void)
{
	guard(SC_MustGetNumber);
	if (SC_GetNumber() == false)
	{
		SC_ScriptError("Missing integer.");
	}
	unguard;
}

//==========================================================================
//
//	SC_CheckFloat
//
//==========================================================================

boolean SC_CheckFloat(void)
{
	guard(SC_CheckFloat);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Float = strtod(sc_String, &stopper);
		if (*stopper != 0)
		{
			SC_UnGet();
			return false;
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
//	SC_GetFloat
//
//==========================================================================

boolean SC_GetFloat(void)
{
	guard(SC_GetFloat);
	char *stopper;

	CheckOpen();
	if (SC_GetString())
	{
		sc_Float = strtod(sc_String, &stopper);
		if (*stopper != 0)
		{
			Sys_Error("SC_GetFloat: Bad floating point constant \"%s\".\n"
				"Script %s, Line %d", sc_String, *ScriptName, sc_Line);
		}
		return true;
	}
	else
	{
		return false;
	}
	unguard;
}

//==========================================================================
//
//	SC_MustGetFloat
//
//==========================================================================

void SC_MustGetFloat(void)
{
	guard(SC_MustGetFloat);
	if (SC_GetFloat() == false)
	{
		SC_ScriptError("Missing float.");
	}
	unguard;
}

//==========================================================================
//
// SC_UnGet
//
// Assumes there is a valid string in sc_String.
//
//==========================================================================

void SC_UnGet(void)
{
	guard(SC_UnGet);
	AlreadyGot = true;
	unguard;
}

//==========================================================================
//
// SC_Check
//
// Returns true if another token is on the current line.
//
//==========================================================================

/*
boolean SC_Check(void)
{
	char *text;

	CheckOpen();
	text = ScriptPtr;
	if(text >= ScriptEndPtr)
	{
		return false;
	}
	while(*text <= 32)
	{
		if(*text == '\n')
		{
			return false;
		}
		text++;
		if(text == ScriptEndPtr)
		{
			return false;
		}
	}
	if(*text == ASCII_COMMENT)
	{
		return false;
	}
	return true;
}
*/

//==========================================================================
//
// SC_MatchString
//
// Returns the index of the first match to sc_String from the passed
// array of strings, or -1 if not found.
//
//==========================================================================

int SC_MatchString(const char **strings)
{
	guard(SC_MatchString);
	int i;

	for (i = 0; *strings != NULL; i++)
	{
		if (SC_Compare(*strings++))
		{
			return i;
		}
	}
	unguard;
	return -1;
}

//==========================================================================
//
// SC_MustMatchString
//
//==========================================================================

int SC_MustMatchString(const char **strings)
{
	guard(SC_MustMatchString);
	int i;

	i = SC_MatchString(strings);
	if (i == -1)
	{
		SC_ScriptError(NULL);
	}
	return i;
	unguard;
}

//==========================================================================
//
// SC_Compare
//
//==========================================================================

boolean SC_Compare(const char *text)
{
	guard(SC_Compare);
	return !stricmp(text, sc_String);
	unguard;
}

//==========================================================================
//
// SC_ScriptError
//
//==========================================================================

void SC_ScriptError(const char *message)
{
	guard(SC_ScriptError)
	const char* Msg = message ? message : "Bad syntax.";
	Sys_Error("Script error, \"%s\" line %d: %s", *ScriptName,
		sc_Line, Msg);
	unguard;
}

//==========================================================================
//
// CheckOpen
//
//==========================================================================

static void CheckOpen(void)
{
	guard(CheckOpen);
	if (ScriptOpen == false)
	{
		Sys_Error("SC_ call before SC_Open().");
	}
	unguard;
}

//**************************************************************************
//
//	$Log: sc_man.cpp,v $
//	Revision 1.10  2006/03/04 16:01:34  dj_jl
//	File system API now uses strings.
//	
//	Revision 1.9  2006/03/02 23:24:35  dj_jl
//	Wad lump names stored as names.
//	
//	Revision 1.8  2005/07/03 12:07:37  dj_jl
//	Fixed opening of lump num scripts.
//	
//	Revision 1.7  2005/05/03 15:00:11  dj_jl
//	Moved switch list, animdefs enhancements.
//	
//	Revision 1.6  2002/01/07 12:16:43  dj_jl
//	Changed copyright year
//	
//	Revision 1.5  2001/10/12 17:31:13  dj_jl
//	no message
//	
//	Revision 1.4  2001/08/04 17:27:39  dj_jl
//	Added consts to script functions
//	
//	Revision 1.3  2001/07/31 17:16:31  dj_jl
//	Just moved Log to the end of file
//	
//	Revision 1.2  2001/07/27 14:27:54  dj_jl
//	Update with Id-s and Log-s, some fixes
//
//**************************************************************************
